/*
 
     Centaurus X-FER  data types and prototypes
*/


// #define DEBUG1  // State machines information
// #define DEBUG2  // High level network information
// #define DEBUG3  // Detailed network information



#ifdef __LINUX__
#define CNT_RESULT_SIZE 128
#define MAX_DEVICES  14
#define MAX_OBJECTS  100
#define MAX_ID       20
#else
#define CNT_RESULT_SIZE 128
#define MAX_DEVICES  1
#define MAX_OBJECTS  1
#define MAX_ID       20
#endif

#define CNT_MSG_PKTSIZE   (128)
#define CNT_MSG_MAXPKTS    (64)

#define SESSION_TTL	300		/* Time for session to live */
#define TTL_START	900
#define TTL_REVIVE      300			/* Below this threshold update
					   if current TTL is divisible by 10 */
#define TTL_DIVISOR	50

/* Device Context Crap */
#define  CNTXT  cntDta.contxts[i].

///  This one is used for IR communications
//typedef  IrDeviceAddr  CntDeviceAddr;

#ifdef UDP
#include <CntXFerUDP.h>
#endif

#ifdef IR

#endif

#ifndef __CNT_IMPLENENTATION_DEFINED
#error "No level1 protocol info provided"
#endif

/***********************************************************************
			CENTAURUS  TYPES
************************************************************************/


extern char _NewPktData[1024 + 4];

#define SessTag (_NewPktData[0])
#define SessNum (_NewPktData[1])
#define SessPktData (&_NewPktData[2])

typedef enum { 
		AskName, 
		FindDevices, 
		WaitForDiscovery, 	
		ValidateDevices, 
		Idle, 
		Error,
	        Comm 
	     }  CntXFerState;

typedef enum { 	
		vlLAPConnReq , 
		vlLAPQuerySent , 
		vlNextDevice , 
		vlDisconnect , 
	        cmStart , 
		cmConReq , 
		cmWaitPkt , 
		cmSend , 
		cmSent , 
		cmAbort  } SubStateType;

typedef enum 
		{ 
		  cntActNone , 
		  cntActSetName , 
		  cntActUpdateDiscovery , 		
	          cntActUpdateDevicelist 
		}  CntActionResult;

typedef enum 
		{ 
			shStart , 
			shWait , 
			shSendCmd , 
			shCmdSent , 
			shCmdBusy , 
			shAckRej , 
			shObjRcvd ,
	       		shProcRcvd , 
	       		shXmit , 
			shXmitBusy, 
			shXmitAbort , 
			shPkSent , 
			shPollRcvd , 
			shDisconnect,
	       		shDoneRcvd , 
			shPkRcvd , 
			shProcSent ,  
			shRecv , 
			shRcvAbort,
	       		shNopeRcvd , 
			shConnctd } shStateType;

typedef  int (*NewObjectCallback) ( char * Name );

typedef struct 
{
    CntDeviceAddr addr;
    char UserName [40];
    int  rlSap;

    int   TTL;
    
    char  Good;
    char  Valid;
    char  New;
} Device;


typedef struct
{
  char PktNum;
  char PktData[CNT_MSG_PKTSIZE]; 
} CntMsgPkt;

typedef struct
{
  int   Overflow;		/* True when there is no more space in the obj */
  int   NxtPtr;			/* Pointer to next char for CntMsgNextChar     */
  int   Size;			/* Size of the message  */
  unsigned char  recvmap [8];   /* 7*8 = 56 packets is max Message, 
			  	   128bytes * 56 = 7K max message */
  char  Id[20];			/* Object name or Id[0] == 0 */
  long   TimeStamp;		/* Stamp of the object */
  CntMsgPkt Packets[CNT_MSG_MAXPKTS];
} CntMsgObj;

typedef struct 
{
  int 		  active;		/* Context is active */
  int 		  deviceIdx;		/* Device Idx of this Context */

  shStateType     shState;		/* State of sh protocol  */
  SubStateType    subState;		/* State of the validation phase */
  int             procExpctd;           /* Proceed Expected            */

  int 		  pktCTS;		/* Packet is free */
  int		  CurrentSession;	/* Current Session number  */
  CntMsgObj	  Send;

  int		  inProgress;		/* Transmission is in progress */
  int 		  SessTTL;		/* Abort transmission timer    */
  int		  SessionNumR;		/* Incoming session number     */
  int		  SessionNumS;		/* Outgoing session number     */
} CntSessContxt;

typedef struct
{
  CntXFerState   State;			/* State of the protocol */

  shStateType     shState;		/* State of sh protocol  */
  SubStateType    subState;		/* State of the valiadation phase */
  int             procExpctd;           /* Proceed Expected            */
  int		  NxtDev;		/* Next device to validate or talk to  */  
  int 		  nDevicesCnt;
  int  		  bDevicesValid;
  
  int 		  DiscoveryTimeout;
  int  		  cmTimeout;		/* Comm timeout */
  int		  SpCurrentDevice;      /* Current Device that we are validating */

  int 		  pktCTS;		/* Packet is free */

  long		  timeTick;		/* Time tick      */


  /* UDP Session data  */

  int 		  badQuality;		/* Bad Communication Quality */
  int		  inProgress;		/* Session is in progress */
  int		  SessTTL;		/* Time to live for this session */
  int		  SessionNumR;		/* Incoming session number     */
  int		  SessionNumS;		/* Outgoing session number     */

  /* Context Stuff */
  int		  currentCntxt;		/* Current Context */
  CntSessContxt   contxts[MAX_DEVICES]; /* Contexts for each device */


  /* Object table  */

  char 		  RcvObjID    [MAX_OBJECTS][MAX_ID];  /* Objects to recieve */
  long		  RcvObjTime  [MAX_OBJECTS];	      /* Timestamps of objects */
  
  /* Incoming Object */
  CntMsgObj	  Rcv	     [MAX_OBJECTS];	      /* Recieve buffer for the obj */
  char		  RcvObjSess [MAX_OBJECTS][MAX_ID];   /* ObjID of obj to Rcv   */ 
  char            RcvUsr     [MAX_OBJECTS][40];	      /* Name of device in Rcv */
  int		  RcvTTL     [MAX_OBJECTS];	      /* Time to live for a session */
  int		  CurrentSession;		      /* Current Session number  */
  int             Completed  [MAX_OBJECTS];	      /* True when session completed */

  /* Outgoing Object */
  CntMsgObj	  Send;
 
  /* Device Tables */
#ifdef SERVER
  int    	  nGoodDevices;
  CntDeviceAddr   NewDevices      [MAX_DEVICES]; /* List of new devices */
  Device          Devices         [MAX_DEVICES];
  char * 	  DeviceList      [MAX_DEVICES];
  int    	  DeviceIdxs      [MAX_DEVICES];

  /// Pointer to the callback function for received objects.....
  /// if callback decides that it can receive this object then
  /// it must add the object through ObjAccept call
  NewObjectCallback newObj; 

#endif
 

} CntData;



/********************************************************************
			CENTAURUS  VARIABLES
********************************************************************/


/* Timeouts */

extern CntData  cntDta;

void 		 CntSetUserName ( char * UserName );
void 		 CntFinish 	( void );
int 		 CntStart       ( void );
CntActionResult  CntXFerAction  ( void );



void SpClearDeviceAddrs 	( void ); 	
void SpLapDisconnectAction 	( void );
void SpDisconnectIrLap 		( void );
void SpPreparePacket ( char * Buffer , int len );



void CntMsgInitObj  ( CntMsgObj * obj , char *Id ) ;
void CntMsgMarkPkt  ( CntMsgObj * obj , int pktNum );
int  CntMsgIsDone   ( CntMsgObj * obj );
char CntMsgNextChar ( CntMsgObj * obj );
void CntMsgAddToObj ( CntMsgObj * obj , char * str );


int  ObjInitObjs      ();
int  ObjSessInProgr   ( char * Id ,  char * Usr );
void ObjStopSess      ( char * Id , char * Usr );
void ObjNewSess       ( char * Id , char * Usr );
int  ObjUpdateTTLs    ();
void ObjStopAccepting ( char * Id );
void ObjAccept        ( char * Id );
int  AcceptableObj    ( char * Id , long Time );
void ObjNewObjSetCb   ( CntData * Dta , NewObjectCallback cb );


int ShXFerAction  ( char * Data , int len );
int ShSendCmd     ( char * Cmd , char *Arg );
int ShSendPkt     ( );
int ShNextToken   ( char * Str );

void MemCopy ( char * Dest , char * Src , int l );




/********************************************************************************
			    Transport Protocol Tokens
*********************************************************************************/

#define CNT_HELO	"Hi!"
#define CNT_HELORESP    "What's up"

#define CNT_OBJ		"Object"
#define CNT_NOPE	"Nope"
#define CNT_POLL	"Poll"
#define CNT_REJ		"Rej"
#define CNT_ACK		"Ack"
#define CNT_PROC	"Proc"
#define CNT_PK		"PK"
#define CNT_ABOR	"Abort"
#define CNT_DONE	"Done"
#define CNT_SENT        "Sent"

