/*************************************************************
File    : CntXFerMsg.c
Contains: Message Handling routines for Centaurus XFer
Author  : Vlad Korolev
Last Mod: Tue Oct  3 12:46:34 EDT 2000
History : 
Tue Oct  3 12:49:30 EDT 2000   Comments added
**************************************************************/

/* Load operating system dependant declarations */
#ifndef __LINUX__
#include <Pilot.h>
#else
#include  <Str.h>
#endif

#include <CntXFer.h>
#include <DebugWnd.h>

/* Macros to find the correct packet in the Bitmap */
#define CntMsgPktNum(n)		(n/128)
#define CntMsgPktOff(n)		(n%128)

/**************************************************************
   Function    :    CntMsgMarkPkt
   Purpose     :    Mark specified packet in the bitmap of the
                    specified object
**************************************************************/
void CntMsgMarkPkt  ( CntMsgObj * obj , int pktNum )
{
    int byte = pktNum / 8;
    int bit  = pktNum % 8;
 
    obj->recvmap[byte] |= ( 1 <<  bit );
}

/**************************************************************
   Function    :    CntMsgNxtChnk
   Purpose     :    FILL_IN
**************************************************************/
int  CntMsgNxtChnk ( CntMsgObj * obj )
{
   int i,j; 

   for ( i = 0 ; i < 8 ; i ++ )
    if ( cntDta.Send.recvmap[i] != 0xff )
       for ( j = 0 ; j < 8 ; j ++ )
	     if ( ! (cntDta.Send.recvmap[i] & ( 1 << j ))) return i*8+j;
   return -1;
}

/**************************************************************
   Function    :    FILL_IN
   Purpose     :    FILL_IN
**************************************************************/
int  CntMsgIsDone  ( CntMsgObj * obj )
{
   int pkts   =  ( (obj->Size+1) / 128 ) + ((obj->Size % 128)?1:0); 
   int wPkts  =  (pkts/8) ;  
   int rPkts  =  pkts % 8;  
   int i;

   int wPktsDone =0 , rPktsDone = 0;

   PRINTF1 ( "M: Sz: %d pk %d Wpk %d rpk %d " , 
	     obj->Size , pkts , wPkts , rPkts );

   for ( i = 0 ; i < wPkts ; i++ ) 
     if ( obj->recvmap[i] != (unsigned char)0xff ) return 0;  
   

   PRINTF1 ( "Wpkts passed" );

   if ( rPkts == 0 ) return -1;

   PRINTF1 ( "Might have rpkts" );

   for ( i = 0 ; i < rPkts ; i++ )
   {

 	PRINTF1 ( "i %d R:%d B %d Cond %d Cn %d " ,i, 
		 (int)obj->recvmap[wPkts], (int)(1<<i ),
		 !(obj->recvmap[wPkts] & ( 1 << i )), 
		  (obj->recvmap[wPkts] & ( 1 << i ))   ); 

 	if (  !(obj->recvmap[wPkts] & ( 1 << i )))  return 0;
   }

   PRINTF1 ( "Rpkts passed" );

   return -1;
}


/**************************************************************
   Function    :    FILL_IN
   Purpose     :    FILL_IN
**************************************************************/
char CntMsgNextChar ( CntMsgObj * obj )
{
   int c;
   if ( obj->NxtPtr > obj->Size ) return 0;

   c = obj->Packets[CntMsgPktNum(obj->NxtPtr)].PktData[CntMsgPktOff(obj->NxtPtr)];
   obj->NxtPtr++;
   return c;
}


/**************************************************************
   Function    : CntMsgInitObj   
   Purpose     : Initialize the data transfer area   
**************************************************************/
void CntMsgInitObj ( CntMsgObj * obj , char * Id ) 
{
   int i;
   obj->Size     = 0;
   obj->Overflow = 0;
   obj->NxtPtr  = 0;
   // obj->TimeStamp = 0; /* Session Objects don't have TimeStamps */
   StrCopy ( obj->Id , Id );
   for ( i = 0 ; i < 8;i++)  obj->recvmap[i] = 0;
   for ( i = 0 ; i < CNT_MSG_MAXPKTS ; i++ ) 
  {
   obj->Packets[i].PktNum = i;
   memset (& obj->Packets[i].PktData[0] , 0, CNT_MSG_PKTSIZE );
  }
}


/**************************************************************
   Function    :    FILL_IN
   Purpose     :    FILL_IN
**************************************************************/
void CntMsgAddToObj ( CntMsgObj * obj , char * str )
{
   int l = StrLen ( str ) , pktFree , w;

   if ( obj->Overflow ) return ;
   if ( obj->Size + l > CNT_MSG_MAXPKTS * CNT_MSG_PKTSIZE - 12 ) 
	{ obj->Overflow = -1;  return ; }

   /* See how much will fit in the starting packet */
   pktFree = 128 - CntMsgPktOff(obj->Size);


   if ( pktFree < l ) w = pktFree; else w = l;

   /* Add this much,  change size */
   StrNCopy ( &obj->Packets[CntMsgPktNum(obj->Size)].PktData
             [CntMsgPktOff(obj->Size)] , str , w );
   obj->Size += w;

   if ( l > w ) CntMsgAddToObj ( obj , str+w );
}


