/*************************************************************
   CntXFerSH		---  Object Handling Functions for Centaurus
**************************************************************/


#ifndef __LINUX__
#include <Pilot.h>
#endif
#include <CntXFer.h>
#include <Str.h>
#include <DebugWnd.h>

/*************************************************************
Function:	ObjInitObjs
Purpose :	Initialize object handling area
**************************************************************/
int ObjInitObjs  ()
{
  int i;
  for ( i = 0 ; i < MAX_OBJECTS ; i++ )
  {
     cntDta.Rcv[i].Id [0] = 0;
     cntDta.RcvObjID  [i][0] = 0;
     cntDta.RcvObjTime[i]    = 0;
     cntDta.Completed [i]    = 0;
  }
}


/*************************************************************
Function	:  ObjSessInProgr
Purpose		:  Check if there is a session for a particular 
		    object and device 
Retrun Value	:  Session index if session exist or -1
*************************************************************/
int ObjSessInProgr (  char * Id ,  char * Usr )
{
   int i;
   for ( i = 0 ; i < MAX_OBJECTS ; i++ )
   {
     if ( StrCompare ( cntDta.Rcv[i].Id , Id )) continue;
     if ( StrCompare ( cntDta.RcvUsr [i] , Usr)) continue;
     return i;
   }
   return -1;
}


/*************************************************************
Function:	ObjInitObjs
Purpose :	Stop session with some object for whatever reason 
		and make the data area occupied by that object
		available for another session
**************************************************************/
void ObjStopSess ( char * Id , char * Usr )
{
   int i;

   i = ObjSessInProgr ( Id , Usr );
   if ( i != -1 )  { cntDta.Rcv[i].Id[0] = 0; }
}


/************************************************************* 
Function:   ObjNewSess
Purpose :   Initiate new session 
*************************************************************/
void ObjNewSess  ( char * Id , char * Usr )
{
   int i;
   for ( i = 0 ; i < MAX_OBJECTS ; i ++ )
     if ( cntDta.Rcv[i].Id[0] == 0 && cntDta.Completed[i] == 0 ) 
     {
	CntMsgInitObj ( &cntDta.Rcv[i] , Id );
        StrCopy ( cntDta.RcvUsr[i] , Usr );
	return;
     }
   
   PRINTF1 ( "Newsession failed" );
}




/************************************************************* 
Function:   ObjUpdateTTLs
Purpose :   Update time to live of the objects and throw away
	    all stale objects
*************************************************************/
int ObjUpdateTTLs ()
{
   int i;
   for ( i = 0 ; i < MAX_OBJECTS ; i++ )
   {
      if ( cntDta.RcvTTL[i] == 0 ) continue;
      cntDta.RcvTTL[i]--; 
      if ( cntDta.RcvTTL[i] <= 0 ) cntDta.Rcv[i].Id[0] = 0;
   }
}

/************************************************************* 
Function:   ObjStopAccepting    
Purpose :   Tell Centaurus XFer Module that we no longer
	    accepting a particular object
*************************************************************/
void ObjStopAccepting ( char * Id )
{
  int i;
  for ( i = 0 ; i < MAX_OBJECTS ; i++ )
     if ( !StrCompare ( cntDta.RcvObjID[i] , Id )) 
	    {  cntDta.RcvObjID[i][0] = 0; break; }
}

/************************************************************* 
Function:  ObjAccept   
Purpose :  Tell Centaurus XFer Module that we now accepting
	:  new Object
*************************************************************/
void  ObjAccept  ( char * Id )
{
  int i;
  for ( i = 0 ; i < MAX_OBJECTS ; i++ )
  {
     if ( cntDta.RcvObjID[i][0] == 0 ) { StrCopy ( cntDta.RcvObjID[i] , Id );
					 cntDta.RcvObjTime[i] = 0;
					 break; }
  }
}

#ifdef SERVER
// TODO: Write Comment
void ObjNewObjSetCb   ( CntData * Dta , NewObjectCallback cb )
{
   Dta->newObj = cb;
}
#endif

/************************************************************* 
Function:   AcceptableObj 
Purpose :   
*************************************************************/
/* Check if this object is acceptable */
int AcceptableObj ( char * Id , long Time )
{
  int i;
  for ( i = 0 ; i < MAX_OBJECTS ; i ++ )
     if ( !StrCompare ( cntDta.RcvObjID[i] , Id )) return i;

  // Well if we are here, then most likeley we do not have any
  // objects to accept... so maybe we should go and check if
  // it is a dynamic object, but we can do it only if there is 	
  // a callback

  // TODO: Put somewhere, that when object is sent with time 0 it means
  // that this object must be accepted unconditionally. [ this is caused
  // by the reset on the other end... also everybody must be sure to set
  // only own objects to time 0

#ifdef SERVER
  if ( cntDta.newObj != NULL )
	{
	   (*cntDta.newObj)(Id);
	    for ( i = 0 ; i < MAX_OBJECTS ; i ++ )
	     if ( !StrCompare ( cntDta.RcvObjID[i] , Id )) return i;
	}
#endif
  
  /// Well if we are here, then nothing will help anymore....
  return -1;
}


