


/*
   Centaurus Data   -- Sush Protocol Implementation

*/


#ifndef __LINUX__
#include <Pilot.h>
#endif


#include <DebugWnd.h>
#include <CntXFer.h>
#include <callback.h>

#ifdef IR
#include <IrLib.h>
#include <CntXFerIR.h>
#endif

/********************
   Static Functions 
********************/
static void SpIasCallback (IrStatus status) ;

/* Device Table Stuff */
void SpDisconnectIrLap ()
{
   IrStatus ret;
   cntDta.subState = vlDisconnect;
   debug_msg("disconnecting IrLap");
   if ( IrIsIrLapConnected ( ir_ref))
   {
    if((ret=IrDisconnectIrLap(ir_ref))!=IR_STATUS_PENDING)
    {
	PRINTF("IrDisconnectIrLap() failed: %s", ir_status_str[ret]);	  
	cntDta.State = Error;
    }
   }
   else
   {
      SpLapDisconnectAction ();
   }
}


/* Initialize Device address table */
void SpClearDeviceAddrs ()
{
   int i;
   cntDta.bDevicesValid = false;
   for ( i = 0 ; i < MAX_DEVICES ; i++ )
   {
      cntDta.Devices[i].addr.u32    =  cntDta.NewDevices[i].u32 = 0;
      cntDta.Devices[i].New         =  
      cntDta.Devices[i].Valid       = 
      cntDta.Devices[i].Good        =  0;
      cntDta.Devices[i].UserName[0] =  0;
   }
}


void SpLapDisconnectAction ()
{
#ifdef SERVER
  debug_msg ( "Sush : LAP disconnect" );
  switch ( cntDta.State )
    {
    case ValidateDevices:
      switch ( cntDta.subState )
	{
	case vlNextDevice:
	case vlLAPConnReq:
	case vlLAPQuerySent:
	case vlDisconnect:
	  cntDta.subState = vlNextDevice;
	  cntDta.NxtDev ++;
	  break;
	default:
	  PRINTF( "Val: Spurious disconect : %d " , cntDta.subState );
	}
      break;
    case Comm:
	cntDta.subState = cmStart;
	cntDta.NxtDev ++;
	break;
    default:
      PRINTF( "Spurious disconect, state : %d " , cntDta.State );
    }
#else
  cntDta.State = Idle;
#endif SERVER
}

void SpSendIASQuery ( char * qString ) 
{
  IrStatus ret;
  /* Send IAS Queries */
  query.queryBuf = qString;
  query.queryLen = StrLen(qString);
  query.resultBufSize = CNT_RESULT_SIZE;
  query.result = result;
  query.callBack = SpIasCallback;
  
  if((ret=IrIAS_Query(ir_ref, &query))!=IR_STATUS_PENDING)
    {
      PRINTF("IrIAS_Query() failed: %s", ir_status_str[ret]);
      SpDisconnectIrLap ();
    }
  cntDta.subState = vlLAPQuerySent;
}

void SpPreparePacket ( char * Buffer , int len )
{
   packet.len  = len;
   packet.buff = Buffer;


}

/*
IAS Callback for Validating peer devices 
*/
static void SpIasCallback(IrStatus status) 
{
  IrStatus ret;
  Byte b;
  int type;


  CALLBACK_PROLOGUE;

//  PRINTF("IAS callback: %s %d", ir_status_str[status],status);

  if((query.retCode)==IAS_RET_SUCCESS) 
  {

    IrIAS_StartResult(&query);
    type=IrIAS_GetType(&query);
    PRINTF ("Type is %d Off %d " , type , query.offset );
    switch(type) 
    {
      case IAS_ATTRIB_MISSING:
        debug_msg("reply: attrib not found");  
	SpDisconnectIrLap ();
        break;

      case IAS_ATTRIB_INTEGER: 
	if ( cntDta.SpCurrentDevice != - 1 )
		 /* Bad but it seems like IAS_GetInteger is broken */
#ifdef __LINUX__
		cntDta.Devices[cntDta.SpCurrentDevice].rlSap = ((char*)query.result)[3];
#else
		cntDta.Devices[cntDta.SpCurrentDevice].rlSap = ((char*)query.result)[6];
#endif
	    else
		PRINTF ("Strange: Got integer without proper cDev ");
	PRINTF ("Remote lSap    : %d ", cntDta.Devices[cntDta.SpCurrentDevice].rlSap );
	SpDisconnectIrLap ();
	break;

      case IAS_ATTRIB_USER_STRING:
	/* Device Confirmed to be Valid */
	{
	  int i,j;
	  int bFound = false;
	  
	  b=IrIAS_GetUserStringCharSet(&query);
	  cntDta.SpCurrentDevice = -1;
	  
	  /* See if this devices is an old device */
	  for ( i =0 ;  i < MAX_DEVICES ; i++ )
	    {
	      if ( !cntDta.Devices[i].Good ) continue;
	      if ( cntDta.NewDevices[cntDta.NxtDev].u32 == cntDta.Devices[i].addr.u32 )
		{
		  cntDta.Devices[i].Good = cntDta.Devices[i].Valid = 1;
		  MemMove ( cntDta.Devices[i].UserName , IrIAS_GetUserString(&query) , 
			    IrIAS_GetUserStringLen(&query) );
		  cntDta.Devices[i].UserName[IrIAS_GetUserStringLen(&query)] = 0;
		  bFound = true; 
		  cntDta.SpCurrentDevice = i;
		  SpSendIASQuery ( rlsap_lmp_query );
		  break;
		}
	    }
	  
	  /* Not found then it is a new device */
	  
	  if ( !bFound )
	    {
	      for ( i = 0 ; i < MAX_DEVICES ; i ++ )
		if ( cntDta.Devices[i].Good == 0 ) { bFound = true ; break ; }
	      
	      if ( bFound ) 
		{
		  cntDta.Devices[i].addr.u32 = cntDta.NewDevices[cntDta.NxtDev].u32;
		  cntDta.Devices[i].New = cntDta.Devices[i].Good = cntDta.Devices[i].Valid = 1;
		  MemMove ( cntDta.Devices[i].UserName , IrIAS_GetUserString(&query) , 
			    IrIAS_GetUserStringLen(&query) );
		  cntDta.Devices[i].UserName[IrIAS_GetUserStringLen(&query)] = 0;
		  PRINTF ( "New User : %s " , cntDta.Devices[i].UserName );
		  cntDta.SpCurrentDevice = i;
		  SpSendIASQuery ( rlsap_lmp_query );
		}
	    }
	}
       break;
    }
  }
  else
    {
      PRINTF("IAS reply: %s\n", ir_ias_return_str[(query.retCode>3)?3:query.retCode]);
      SpDisconnectIrLap ();
    }
  
  CALLBACK_EPILOGUE;
}


void CntFinish ( void )
{
    IrClose(ir_ref); 
}



void CntSetUserName ( char * UserName )
{
   StrCopy(attr_username+3,UserName);
   attribs[1].valLen=StrLen(attr_username+2)+2; /* Length of the value. */
   attr_username[2] = (char)(StrLen(UserName));

   StrCopy(dev_name+2,UserName);
   if(ir_ref )
	if (IrSetDeviceInfo(ir_ref, dev_name, 10)!=IR_STATUS_SUCCESS) 
   	{ debug_msg("IrSetDeviceInfo() failed"); return ; }
}


static void InitCntData ()
{
  int i;

  cntDta.State = AskName;

  /* Initialize validation data */
  cntDta.subState = vlNextDevice;
  cntDta.NxtDev = 0;
  cntDta.nDevicesCnt     = 0;
  cntDta.bDevicesValid    = false;
  cntDta.DiscoveryTimeout = 3;
  cntDta.timeTick  = 0;

  ObjInitObjs  ();

}

int CntStart ( void )
{

  IrStatus ret;

  /* open IR library */
  if(SysLibFind(irLibName, &ir_ref)) 
  { 
    PRINTF("SysLibFind(%s) failed", irLibName);
    return -1;
  } 

 InitCntData ();

 PRINTF("got IR library reference: %d", ir_ref);

 /* Open up the IrLibrary */
 if(IrOpen(ir_ref, 0)) 
	{ debug_msg("IrOpen() failed"); return -1; } 

 /* Connection object to the stack */
 if(IrBind(ir_ref, &con, IrCallback)!=IR_STATUS_SUCCESS) 
	{ debug_msg("IrBind() failed"); return -1; } 

 /* Set up device info */
 if(IrSetDeviceInfo(ir_ref, dev_name, 10)!=IR_STATUS_SUCCESS) 
   	{ debug_msg("IrSetDeviceInfo() failed"); return -1; }

 ias_object.attribs = attribs;

 attribs[0].name=attr_name[0];         /* Pointer to name of attribute */
 attribs[0].len=StrLen(attr_name[0]);  /* Length of attribute name */
 attribs[0].value=attr_rsap;          /* Hardcode value (see below) */
 attribs[0].valLen=5;                  /* Length of the value. */

 attribs[1].name=attr_name[1];         /* Pointer to name of attribute */
 attribs[1].len=StrLen(attr_name[1]);  /* Length of attribute name */
 attribs[1].value=attr_username;       /* Hardcode value (see below) */
 attribs[1].valLen=StrLen(attr_username+2); /* Length of the value. */

 attr_rsap[4]=con.lLsap;

 PRINTF ( "Local Lsap is %d %d" , con.lLsap , attr_rsap[4] );

 // set up our service in IAS db
 if((ret=IrIAS_Add(ir_ref, &ias_object))!=IR_STATUS_SUCCESS)
	{ PRINTF("IrIAS_Add() failed: %s", ir_status_str[ret]); return -1; }

 /* Clean up device table */
 SpClearDeviceAddrs ();

 return false;

error:
   return true;    
}






CntActionResult CntXFerAction ( )
{

  int ret;
  CntActionResult actionResult = cntActNone;
  static int pstate = -1;

#ifdef __LINUX__
  if ( cntDta.State != pstate ) 
  {
    PRINTF ( "State %d " , cntDta.State );
    pstate = cntDta.State;
  }
#endif

  switch ( cntDta.State )
  {
#ifdef SERVER
  case Idle:
    cntDta.DiscoveryTimeout--;
    if ( cntDta.DiscoveryTimeout < 1)
      {
	cntDta.State = FindDevices;
	cntDta.DiscoveryTimeout = 3;
      }
    actionResult = cntActUpdateDiscovery;
    break;
  case FindDevices:


    /*** Check if the media is busy */
    if ( IrIsMediaBusy (ir_ref) )  { break; }

    /*** Media is not busy, send search request and then go to Wait for Reply state
	    TODO:  Set up the timer in case nothing happens */
    if((ret=IrDiscoverReq(ir_ref, &con))!=IR_STATUS_PENDING)
      {
	PRINTF("IrDiscoverReq() failed: %s", ir_status_str[ret]);
	cntDta.State = Idle;
	break;
      }
 
    cntDta.State = WaitForDiscovery;
    break;

  case WaitForDiscovery:
	break;
  case ValidateDevices:
    switch ( cntDta.subState )
      {
      case vlLAPConnReq:
	break;
      case vlLAPQuerySent:  
	if (! IrIsIrLapConnected ( ir_ref )) SpLapDisconnectAction ();
	break;

      case vlNextDevice: 
	PRINTF ( "Validating Device %d of %d" , cntDta.NxtDev+1 , cntDta.nDevicesCnt );
	if ( cntDta.NxtDev >= cntDta.nDevicesCnt ) 
	  {	
	    int i;
	    /* Finished --- Construct the list and such */
	    
	    /* Construct lists */
	    cntDta.nGoodDevices = 0;
	    for ( i = 0 ; i < MAX_DEVICES ; i ++ )
	      {
		if ( cntDta.Devices[i].Good == 1 &&  cntDta.Devices[i].Valid  == 1 )
		  {
				   cntDta.DeviceList[cntDta.nGoodDevices] =
                                          cntDta.Devices[i].UserName ;
				   cntDta.DeviceIdxs[cntDta.nGoodDevices] = i;
				   cntDta.nGoodDevices++;
		  }
	      }
	     actionResult    = cntActUpdateDevicelist;
	     cntDta.State    = Comm;
	     cntDta.subState = cmStart;
	     cntDta.NxtDev   = 0;
	     cntDta.DiscoveryTimeout = 3;
	   }
	   else
	   {
	     if ( IrIsMediaBusy ( ir_ref )) break ;

	     cntDta.SpCurrentDevice = -1;

	     /* Send query to the next device */
	     PRINTF ( "Connecting to %lx" , cntDta.NewDevices[cntDta.NxtDev].u32 );
	     if(
		(ret=IrConnectIrLap(ir_ref, cntDta.NewDevices[cntDta.NxtDev]))!=IR_STATUS_PENDING)
	       {
		 PRINTF("IrConnectIrLap() failed: %s", ir_status_str[ret]);
		 cntDta.NxtDev++;   
	       }
	       else cntDta.subState = vlLAPConnReq;
	   }
	   break;
	}
    break;
 
#else
  case Idle:	
	break;
#endif 
  case  AskName:
	if ( attr_username[3] == 0) 
		actionResult = cntActSetName; 
	   else  
		{ 
		  int i;
		  CntMsgInitObj ( &cntDta.Send , "Hello" );
 		  for ( i = 0 ; i < 25 ; i ++ )
 		  {
			CntMsgAddToObj ( &cntDta.Send , "Hello there.\nI am " );
			CntMsgAddToObj ( &cntDta.Send ,  attr_username+3      );
			CntMsgAddToObj ( &cntDta.Send , " :-) \n" );
			cntDta.Send.TimeStamp = cntDta.timeTick+1;
		  }
		  ObjAccept ( "Hello" );
		  cntDta.State = Idle;
		}
	break;

  case  Comm:
    switch ( cntDta.subState )
      {
      case cmSend:
	// if ( IrIsMediaBusy ( ir_ref )) break ;
	cntDta.cmTimeout = 30;
	if( IR_STATUS_PENDING !=
	    (ret=IrDataReq (ir_ref, &con , &packet )))
	  {
	    PRINTF("IrDataReq() failed: %s", ir_status_str[ret]);
	    cntDta.subState = cmAbort; 
	  }
	cntDta.pktCTS = 0;
        cntDta.subState = cmSent;
	break;
      case cmStart:
    	if ( IrIsMediaBusy ( ir_ref )) break ;
	PRINTF ( "Connecting to device  %d of %d ", cntDta.NxtDev+1 , cntDta.nGoodDevices );
	// Check if there are more devices to connect to
	if ( cntDta.NxtDev < cntDta.nGoodDevices ) 
	  {
	    
	    cntDta.cmTimeout = 30;
	    if(
	       (ret=IrConnectIrLap(ir_ref, cntDta.Devices
				   [
				    cntDta.DeviceIdxs [cntDta.NxtDev]
				   ].addr  ))!=IR_STATUS_PENDING)
	      {
		PRINTF("IrConnectIrLap() failed: %s", ir_status_str[ret]);
		cntDta.NxtDev++;   
	      }
	    else cntDta.subState = cmConReq;
	  }
	else
	  {
	    cntDta.State = Idle;
	  }
	
	break;
      case vlDisconnect:
	break;
      case cmAbort:
	 PRINTF ( "Aborting ...." );
	 SpDisconnectIrLap ();
	 break;
      case cmSent:
      case cmWaitPkt:
      case cmConReq:
	// Check  timeout
	// Disconnect if time is up
	if ( cntDta.cmTimeout-- < 1 ) { debug_msg("Timeout Expired"); SpDisconnectIrLap ();}
	break;
      default:
	  PRINTF ( "Panic unknown comm state %d" , cntDta.subState );
      }
    break;
  default:
    PRINTF( "Don't know action for state %d" , cntDta.State );
  }

  return actionResult;
}

void IrCallback(IrConnect *connect, IrCallBackParms *parms) 
{
  IrStatus ret;
  int i;
  int max_tx;
  CALLBACK_PROLOGUE;

  PRINTF("Event: %s", ir_event_str[parms->event]);

  switch(parms->event) 
    {
    case LEVENT_STATUS_IND:
      PRINTF("Status = %s", ir_status_str[parms->status]);
      if( parms->status == IR_STATUS_NO_PROGRESS ) SpDisconnectIrLap ();
      break;

    case LEVENT_DISCOVERY_CNF:

      if ( cntDta.State !=  WaitForDiscovery ) break;
      /* Clear Devices New and Valid Bits */
      for ( i = 0 ;i < MAX_DEVICES ; i++ )
      {
	 cntDta.Devices[i].New = cntDta.Devices[i].Valid = 0;
      }
      cntDta.nGoodDevices = cntDta.nDevicesCnt = 0;
      if(parms->deviceList->nItems > 0) 
      {
 
	for(i=0;i<parms->deviceList->nItems;i++)
  	{
	  PRINTF("%d: %lx.%d.%d: %s",i, 
		 parms->deviceList->dev[i].hDevice.u32,
		 parms->deviceList->dev[i].xid[0],
		 parms->deviceList->dev[i].xid[1],
		 &parms->deviceList->dev[i].xid[2]);
	  if ( i > MAX_DEVICES ) break;
	  
	  cntDta.NewDevices[i].u32 = parms->deviceList->dev[i].hDevice.u32;
	  cntDta.nDevicesCnt ++;
	}

        /* Confirm old devices */
	 for ( i = 0 ; i < MAX_DEVICES ; i ++ )
         {
	    int j , bFound = false;
	    for ( j = 0 ; j < cntDta.nDevicesCnt ; j++ )
	    {
		if ( !cntDta.Devices    [i].Good ) continue;
		if (  cntDta.NewDevices [j].u32 == cntDta.Devices[i].addr.u32 ) 
						      { bFound = true; break; }
	    }
 	    if ( !bFound )  cntDta.Devices[i].Good = 0; 
	    else PRINTF ( "Devices %lx still here " , cntDta.Devices[i].addr.u32 );
         }
	cntDta.State = ValidateDevices;
	cntDta.NxtDev = 0;
	cntDta.subState = vlNextDevice; 
      } 
      else 
      {
	debug_msg("no IrDA devices in range");
	cntDta.nGoodDevices = 0;
	cntDta.State = Idle;
      }
      break;

    case LEVENT_LM_CON_IND:
      debug_msg("accepting...");
	/* accept connection */
      
      StrCopy ( PktData , CNT_HELORESP );
      packet.buff = (char*)PktData;
      packet.len = StrLen (PktData ) + 1;
      

      PRINTF("received %d bytes", parms->rxLen);
      hex_dump(parms->rxBuff, parms->rxLen);

      IrSetConTypeLMP ( &con );
      if((ret=IrConnectRsp(ir_ref, &con, &packet, 0x7))!=IR_STATUS_PENDING)
	{ PRINTF("IrConnectRsp() failed: %s",ir_status_str[ret]);
	   SpDisconnectIrLap (); }

      cntDta.State     = Comm;
      cntDta.subState  = cmWaitPkt;
      cntDta.shState   = shStart;
      cntDta.cmTimeout = 30;

      ShXFerAction ( parms->rxBuff , parms->rxLen );

      break;

    case LEVENT_LAP_DISCON_IND:
	SpLapDisconnectAction ();
	break;

    case LEVENT_LAP_CON_CNF:
      if ( cntDta.State == ValidateDevices && cntDta.subState == vlLAPConnReq )
      {
	PRINTF ( "LAP Connection confirmed for %d ", cntDta.NxtDev );
	SpSendIASQuery ( rlsap_usr_query );
      }
      if ( cntDta.State == Comm && cntDta.subState == cmConReq )
      {
	int dIdx = cntDta.DeviceIdxs[cntDta.NxtDev];
	PRINTF ( "LAP Connection confirmed for %d ", cntDta.NxtDev ); 
	// Open connection to new device
	StrCopy ( PktData , CNT_HELO );
	StrCat  ( PktData , " " );
	StrCat  ( PktData , cntDta.Devices[dIdx].UserName );
	packet.buff = (char*)PktData;
	packet.len = StrLen (PktData )+1;
	con.rLsap = cntDta.Devices[dIdx].rlSap ;
	IrSetConTypeLMP ( &con );
	if ( IR_STATUS_PENDING == 
	     IrConnectReq    ( ir_ref , &con , &packet , 25 ))
	  { cntDta.subState = cmConReq; cntDta.cmTimeout = 30;}
	else
	  { SpDisconnectIrLap (); }
      }
      break;
    
    case LEVENT_DATA_IND:
      PRINTF("received %d bytes", parms->rxLen);
      cntDta.subState = cmWaitPkt; cntDta.cmTimeout = 30;
      ShXFerAction ( parms->rxBuff , parms->rxLen );
      CntXFerAction ();
      break;

    case LEVENT_LM_CON_CNF:
      max_tx = IrMaxTxSize(ir_ref, &con); 
      PRINTF("max tx size: %d ", max_tx); 
      cntDta.cmTimeout = 30; cntDta.subState = cmWaitPkt;
      cntDta.shState   = shStart;

      hex_dump(parms->rxBuff, parms->rxLen);

      ShXFerAction ( parms->rxBuff , parms->rxLen );

      /* continue with PACKET_HANDLED */
    case LEVENT_PACKET_HANDLED:
      cntDta.pktCTS = -1;
      if ( cntDta.subState == cmSent ) 
      { 
        cntDta.cmTimeout = 30; cntDta.subState = cmWaitPkt; 
	ShXFerAction ( "Sent" , 4 );
      }

      break;
    }
  CALLBACK_EPILOGUE;
}



