/********************************************************************************
File	:	XmlParse.c
Purpose :	General support for Parsers

********************************************************************************/


#ifdef PALM_PILOT
   #include <Pilot.h>
   #include <IrLib.h>
   #include <CntXFer.h>
   #define  strlen StrLen
#endif

#include "XmlParse.h"

/* Entity related stuff */
EntEntry Entities  [MAX_ENTITY];		/* Enitity data storage  */
int      freeEnt = 0;				/* Next available entity */
int      currentEnt = -1;			/* Current Enity that we are writing into */
int 	 currentEntNamePos = -1;		/* Current Position in the entity name   */
int 	 currentEntValPos  = -1;		/* Current Position in the entity value  */


/*****************************************
Function	:	Error
Purpose		: 	Print out an error and quit 
******************************************/
// void Error ( char * st ) {puts ( st );exit(1);}




/*****************************************
Function	:	matchInput
Purpose		: 	Given a state and an input character, perform go to 
			next state according to the input or return error
******************************************/
int  matchInput (int state , int c)
{
   int i;

//   printf ( " %d  %c \n" , state ,  c );

#define CCond  States[state].conds[i]

   /* Go through all conditions for current state */
   for ( i = 0 ; CCond.inp != 0 ; i++ )
   {
      int  condResult = 0;

      switch ( CCond.k )
      {
	case match:
	    if ( CCond.inp == c ) condResult = -1;
	    break;

	case notmatch:
	    if ( CCond.inp == c ) condResult = -1;
	    break;

	case any:
	      condResult = -1;
	      break;

	case space:
	    switch ( c )
	    {
		case ' ':
		case '\n':
		case '\r':
		case '\t':
			condResult = -1;
			break;
		default:
			condResult = 0;

	    }
	    break;
      }

      /* If condition matched, then go to the next state */
      if ( condResult ) { return  CCond.goTo; }

#ifdef TRACE_PARSER
      printf ( "Matching  %d %c %d %d\n" , CCond.k , CCond.inp , CCond.goTo , condResult );
#endif

   }

   return -1;

#undef CCond
}

static int s   = 0;
static int ln  = 0;
static int col = 0;

/*****************************************
Function	:	ParseReset
Purpose		: 	Reset the paser
******************************************/
void ParseReset ()
{
  ln = 0; col = 0; s = 0;

}



/*****************************************
Function	:	Error
Purpose		: 	Print out an error and quit 
******************************************/
xmlParseResult XmlParseChar ( int c )
{

  
  /* Increase column counter */
  col++;

  /* Increase line counter and reset column counter */
  if ( c == '\n' ) { ln++; col = 0; }
  
  /* Match current input with the current conditions */
  s = matchInput ( s , c );
  
#ifdef TRACE_PARSER
  printf ( "State %d\n" , s );
#endif


  /* If current input was wrong return parse error */
  if  ( s == -1 ) 
    { 
      printf ( "%d,%d :Parse error, State(%d) \n", ln+1 , col , s ); 
      return xmlParseError; 
    }
  
  /* Perform desired action for this state and return it's result */
  if ( States[s].action != 0 ) return  XmlParseAction ( States[s].action , c);
  
  /* No action is needed and everything is okay */
  return xmlParseOk;


}


#ifdef PALM_PILOT

static CntMsgObj * cMsg = NULL;
void SetCurrentMsg ( CntMsgObj * msg )
{
    cMsg = msg;

}


#define getchar()  CntMsgNextChar(cMsg)



#endif


/*****************************************
Function	:	NextChar
Purpose		: 	Get the next character for processing
			This function handles ENITITIES, but it
			is not necessary, and also ENTITIES are not
			fully supported right now.
******************************************/
int NextChar ()
{

  static char nEntName[MAX_ENT_NAME];
  static char nBuff[1024];
  static int  bPtr = -1;
  static int  nEntNameIdx;




  int c;
  
#ifdef PALM_PILOT
    if ( cMsg == NULL ) return 0;
#endif

  /* If buffer is empty read the character from stdin */
  if ( bPtr == -1 ) c = getchar(); else c = nBuff[bPtr--];
  
  /* Looks like an enitity, if it is investigate it more
     and act accordingly */
  if ( c == '%' ) 
    { 
      int d; 
      nEntNameIdx = 0;
      d = getchar (); 
      if ( d == ' ' ) 
	{ 
	  nBuff[++bPtr] = ' ';  
          c = '%';
	}
	else
	{
 	  int i;
	  nEntName[nEntNameIdx++] = d;
	  while ( (d = getchar()) != ';' )
	    nEntName[nEntNameIdx++] = d;
	  nEntName[nEntNameIdx++] = 0;
	  

	  for ( i = 0 ; i < freeEnt ; i ++ )
	    {
	      if ( !strcmp ( Entities[i].EntName , nEntName ))
		{
		  int q = strlen(Entities[i].EntVal )-1;
		  while ( q >= 0 ) nBuff[++bPtr] = Entities[i].EntVal[q--];
		  return NextChar ();
		  break; 
		}
	    }
	   
	}
      
    }

   /* Return current character */

  return c;
}

