#include <Pilot.h> 
#include <SysEvtMgr.h> 
#include <NetMgr.h>
#include <Unix/sys_types.h>
#include <Unix/sys_socket.h>
#include <StringMgr.h>
#include "Minstrel.h"
#include "MData.h"
#include "MDataRes.h"


/* Network specific things */
Word AppNetRefnum;
Long AppNetTimeout = 150;
Err  errno;
int  networkOpen = 0;

/**************************************************
This will go to the Ministrel Support library
**************************************************/

static char PrintTmp[128];
#define FldPrintF(id,format,args...) {StrPrintF(PrintTmp,format, ## args);Say(id,PrintTmp);}


MinstrelData Data ;



typedef struct 
{ 
  Byte ReplaceMe;
} AppPreferenceType  ;

const unsigned long  AppFileCreator = APPID;

static Word appPrefID = 1;
static Word appPrefVersionNum = 1;
static AppPreferenceType prefs;



void Say ( int Id , CharPtr text );

void NetError (char *One, char *Two)  
{
  FrmCustomAlert (NetErrorAlert, One != NULL ? One : " ",  
		  Two != NULL ? Two : " ", " ");
}

int PrepareNetwork ()
{
  Byte allUp = false;
  Word ifErrors = -1;
  int  Err;

  if ( networkOpen ) return;

  Err = SysLibFind  ("Net.lib", &AppNetRefnum);
            
  if (Err)
    {
      ErrDisplay ( "No network library" );
      return 0;
    }
    
  NetLibOpen ( AppNetRefnum , &ifErrors );
  Err = NetLibConnectionRefresh ( AppNetRefnum , true  ,
				  &allUp , &ifErrors );    

  if ( Err != 0 )
    {
      NetError ( "Connection Refresh Failed" , NULL  );
      return 0;
    }

  if ( ifErrors != 0 )
    {
      char Err[10];
      StrPrintF ( Err , "%d" , ifErrors );
      NetError ( "Interface Errors detected " , NULL  );
      return 0;
    }
  if ( allUp == 0 )
    {
      NetError ( "NotAll interfaces are up "  , NULL  );
      return 0;
    }

  if ( MinstrelInit (&Data) == 0 ) networkOpen = -1;
  return -1;
}


static void UIBlink ( int Start , int End )
{
  static bMainFrmUpd;

  if ( bMainFrmUpd  )
    {
      bMainFrmUpd = false;
      WinDrawLine ( Start , 5 , End , 5 );
    } 
  else
    {
      bMainFrmUpd = true;
      WinEraseLine ( Start , 5 , End , 5 );
    }
}

void DumpStrExt ( char *Tmp , char * Dta , int n , int spaces );

/********************************************************
Preferences Save and Restore Functions
 ********************************************************/
void SavePreferences ()
{
  PrefSetAppPreferences (AppFileCreator,
                         appPrefID, appPrefVersionNum,
                         &prefs, sizeof (prefs), true);
}

void UpdatePreferences ( int PktNum , int PktSize )
{
  SavePreferences ();
}

void RestorePreferences ()
{
  Word prefsSize;

  if ( noPreferenceFound != 
       PrefGetAppPreferences ( AppFileCreator , appPrefID ,
			       NULL , &prefsSize , true ))
    {
      PrefGetAppPreferences (AppFileCreator, 
			     appPrefID, 
			     &prefs, 
			     &prefsSize , 
			     true); 
    
    }

}


static VoidPtr GetObjectPtr(Word objectID)
{
  FormPtr frmP;


  frmP = FrmGetActiveForm();
  return (FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID)));

}

void TestSend (int n)
{
  // Send bunch of packets somewhere 
  int i;
  NetIPAddr addr;
  struct sockaddr_in  Sockt;
  int lSock;
  

  Sockt.sin_family = AF_INET;
  Sockt.sin_port   = htons (1024);
  
  addr = NetLibAddrAToIN(AppNetRefnum , "130.85.100.127" );
  MemMove ( (char *) &Sockt.sin_addr.s_addr, &addr, 4);
  
  lSock = socket (AF_INET, SOCK_DGRAM, 17);
  connect (lSock , &Sockt , sizeof (Sockt));
  
  for ( i = 0 ; i < 30 ; i ++ )
    {
      static char Str[] = "Some random stuff is getting sent"
	"The More junk the better we all feel about not cleaning up"
	"This packet will self destruct in 15 minutes unless you do it";
      send ( lSock , Str , sizeof(Str) , 0 );
    }
  close ( lSock );
}
	    


void Say ( int Id , CharPtr text )
{
  FormPtr frmP;
  FieldPtr fld;
  CharPtr  newPtr;
  VoidHand newHand;
  VoidHand oldHand;
  Word  objIdx;

  frmP = FrmGetActiveForm();
    

  objIdx = FrmGetObjectIndex ( frmP , Id );

  {
    char Tmp[128];
    StrPrintF( Tmp , " Idx %d " , Id );
    if ( objIdx == -1 ) NetError ( "Not in Form  " , Tmp );
  }

  if ( objIdx == -1 )  return;

  fld = (FieldPtr) FrmGetObjectPtr ( frmP , objIdx );
					 
  if ( fld == NULL ) return;

  newHand = MemHandleNew (460);
  newPtr = MemHandleLock (newHand);
  StrCopy  ( newPtr , text );
  MemHandleUnlock ( newHand );
  oldHand = (VoidHand)FldGetTextHandle ( fld );
  FldSetText ( fld  , newHand , 0 , 460 );
  FldDrawField ( fld );
  if ( oldHand != NULL ) { MemHandleFree ( oldHand ); }
}

#define GPHTOP 20
#define GPHBOT 140

static void VGraphDrawDashDash   ( int xS ,  int yS )
{
  if ( yS % 8 < 4 )  
    WinDrawLine ( xS , yS , xS , yS);
}

static void HGraphDrawDashDash   ( int xS ,  int yS )
{
  if ( xS % 8 < 4 )  
    WinDrawLine ( xS , yS , xS , yS);
}

static void VGraphDrawDotDot   (int xS ,  int yS )
{
  if ( yS % 4 < 2 ) 
    WinDrawLine ( xS , yS , xS-1 , yS );
}

static void HGraphDrawDotDot   (int xS ,  int yS )
{
  if ( xS % 4 < 2 ) 
    WinDrawLine ( xS , yS , xS , yS-1);
}


static void VGraphDrawThick    (int xS ,  int yS )
{
  WinDrawLine ( xS , yS , xS-1 , yS );
}

static void HGraphDrawThick    (int xS ,  int yS )
{
  WinDrawLine ( xS , yS , xS , yS-1 );
}

static void VGraphDrawNormal   ( int xS , int yS )
{
  WinDrawLine ( xS , yS , xS , yS );

}

static void HGraphDrawNormal   ( int xS , int yS )
{
  WinDrawLine ( xS , yS , xS , yS );
}

static void HGraphDrawSquare ( int xS , int yS )
{
  WinDrawLine ( xS-1 , yS-1 , xS+1 , yS-1 );
  WinDrawLine ( xS+1 , yS-1 , xS+1 , yS+1 );
  WinDrawLine ( xS+1 , yS+1 , xS-1 , yS+1 );
  WinDrawLine ( xS-1 , yS+1 , xS-1 , yS-1 );
}

static void VGraphDrawSquare ( int xS , int yS )
{
  HGraphDrawSquare ( xS , yS );
}


typedef void LnDrawFunc(int,int);


static void GraphDrawDisjoint ( int xS , int xE , int yS , LnDrawFunc What )
{
    What(xS,yS);
}

static void GraphDrawStraight ( int xS , int xE , int yS , LnDrawFunc What )
{
  while ( xS++ < xE )
    What(xS,yS);
}

static void GraphDrawLegend ()
{
  RectangleType r;
  int    yLbl[4];
  char   *Lbls[4] = { "RSSI" , "Busy" , "Err " , "rErr" };

  int    DisJoint[4] = { 0,1,0,0};
  LnDrawFunc* lnType[4] = {HGraphDrawThick, HGraphDrawSquare,
			   HGraphDrawDashDash , HGraphDrawDotDot };
  int    i;

  r.topLeft.x = 160 - 53;
  r.topLeft.y = GPHTOP+3;
  r.extent.x  = 50;
  r.extent.y  = 40;

  WinEraseRectangle ( &r , 0 );
  WinDrawRectangleFrame  ( simpleFrame , &r  );
  for ( i = 0 ; i < 4 ; i++ )
    {
      yLbl [i] = r.topLeft.y + 9*i;
      if ( !DisJoint[i] )
	GraphDrawStraight ( 160-53+2  , 160-53+18 , yLbl[i]+6, lnType[i] );
      else
	GraphDrawDisjoint ( 160-53+10 , 160-53+18 , yLbl[i]+6, lnType[i] );

      WinDrawChars ( Lbls[i] , 4, 160-53+22 , yLbl[i] );
    }
}


static void GraphDrawPaper ()
{
  int x , y;
  // Draw Box
  WinDrawLine ( 0   , GPHTOP   , 158 , GPHTOP );
  WinDrawLine ( 158 , GPHTOP   , 158 , GPHBOT );
  WinDrawLine ( 158 , GPHBOT   , 0   , GPHBOT );
  WinDrawLine ( 0   , GPHBOT   , 0   , GPHTOP );

  WinDrawLine ( 1   , GPHTOP+1 , 159 , GPHTOP+1 );
  WinDrawLine ( 159 , GPHTOP+1 , 159 , GPHBOT-1 );
  WinDrawLine ( 159 , GPHBOT-1 , 1   , GPHBOT-1 );
  WinDrawLine ( 1   , GPHBOT-1 , 1   , GPHTOP+1 );
  // Draw GRID
  for ( y = GPHTOP ; y < GPHBOT ; y += 4 )
    for ( x = 20 ; x < 159 ; x+= 20 )
      WinDrawGrayLine ( x , y , x , y );
  for ( y = GPHTOP ; y < GPHBOT ; y += 20 )
    for ( x = 2 ; x < 159 ; x+= 4 )
      WinDrawGrayLine ( x , y , x , y );
  // Draw Legend  
  GraphDrawLegend ();
}

static Boolean GraphFormHandleEvent (EventPtr eventP)
{
  Boolean handled;
  FormPtr frmP;
  static ULong lastTick = 0;
  int    DisJoint[4] = { 0,1,0,0};

  CALL_PROLOGUE 
    handled = false;

  switch (eventP->eType) 
    {
    case frmOpenEvent:
      frmP = FrmGetActiveForm();
      FrmDrawForm ( frmP);
      Data.ready = -1;
      handled = true;
      GraphDrawPaper ();
      break;
    case ctlSelectEvent:
      frmP = FrmGetActiveForm();
      switch ( eventP->data.ctlSelect.controlID )
	{
	case ChgFormLeft:
	  FrmGotoForm ( StatForm );
	  break;
	case ChgFormRight:
	  FrmGotoForm ( MainForm );
	  break;
	case GraphClear:
	  MinstrelClearSNDCP ( &Data );
	  MinstrelResetStats ( &Data );
	  break;
	case GraphFlash:
	  MinstrelFlashLED ( &Data );
	  break;
	}
    case nilEvent:
      /* Collect the data and display it */
      UIBlink ( 140 , 150 );

      {
	ULong cTick = TimGetTicks ();
	static int sndTout = 0;
	static int cPos    = 4;
	if ( cTick - lastTick > SysTicksPerSecond() )
	  {
	    /* Do measurement, perform send */
	    lastTick = cTick;

	    if ( sndTout == 2 ) MinstrelAutoRequest ( &Data , 1 , 100 );
	    /* Send out only when we have a channel */
	    if ( sndTout == 5 &&  Data.RRMState == 4 ) TestSend ( 2 );


	    if ( sndTout++ > 5 ) sndTout = 0; 
	    /* Draw Chennel Status */
	    if ( Data.RRMState== 4 ) 
	      {
		WinEraseChars (  "Srch" , 4,  20 , GPHTOP-7 );
		WinDrawChars  (  "Chnl" , 4,  20 , GPHTOP-7 );
	      }
	    else
	      {
		static int b = 0;
		WinEraseChars (  "Srch" , 4,  20 , GPHTOP-7 );
		WinEraseChars (  "Chnl" , 4,  20 , GPHTOP-7 );
		if ( b ) WinDrawChars ( "Srch" , 4 , 20 , GPHTOP-7 );
		b = b ? 0 : 1;
	      }

	    /* Draw Points */
	    if ( sndTout == 3 ) MinstrelFlashLED ( &Data ); 

	    /* Draw RSSI, MDBSBusy , BLER , rBLER */
	    if ( cPos < 160 )
	      {
		int  c;
		int sZ = GPHBOT - GPHTOP;

		char * Vals [4] = { &Data.RSSI , &Data.MDBSBusy ,
				    &Data.BLER , &Data.rBLER };
		static char   pVals[4] = { -1,-1,-1,-1};
		LnDrawFunc*   hDrawFnc[4] = { HGraphDrawThick , 
					      HGraphDrawSquare ,
					      HGraphDrawDashDash, 
					      HGraphDrawDotDot };
		LnDrawFunc*   vDrawFnc[4] = { VGraphDrawThick , 
					      VGraphDrawSquare ,
					      VGraphDrawDashDash, 
					      VGraphDrawDotDot };
	      

		for ( c = 0 ; c < 4 ; c++ )
		  {
		    Word yPos = (*Vals[c]*sZ)/120;
		    if ( c == 3  && *Vals[c] == -1 ) { pVals[c] = -1; continue;}
		    (hDrawFnc[c]) ( cPos , GPHBOT - 1 - yPos );
		    if ( pVals[c] != -1 && ( pVals[c] - yPos > 2 ||
					     yPos - pVals[c] > 2 ))
		      {
			int i,d;
			if ( pVals[c] > yPos ) d = -1 ; else d = 1;
			while ( !DisJoint[c] && yPos != pVals[c]  )
			  (vDrawFnc[c]) ( cPos , GPHBOT - 1 - (pVals[c] += d) );
		      }
		    else pVals[c] = yPos;
		  }
	      

		if ( cPos > 153 )  SndPlaySystemSound ( cPos - 153 );
		else  EvtResetAutoOffTimer ();
		cPos++;
	      }
	  }
      }
   
      /* Draw Legend */

      /* Draw Current Point */
         
      /* RSSI */

      /* Busy */

      break;
    
    }

  CALL_EPILOGUE;
  return handled;
}


static Boolean StatFormHandleEvent (EventPtr eventP)
{
  Boolean handled;
  FormPtr frmP;

  CALL_PROLOGUE 
    handled = false;

  switch (eventP->eType) 
    {
    case frmOpenEvent:
      frmP = FrmGetActiveForm();
      FrmDrawForm ( frmP);
      Data.ready = -1;
      handled = true;
      break;
    case ctlSelectEvent:
      frmP = FrmGetActiveForm();
      switch ( eventP->data.ctlSelect.controlID )
	{
	case ChgFormLeft:
	  FrmGotoForm ( ModelRevForm );
	  break;
	case ChgFormRight:
	  FrmGotoForm ( GraphForm );
	  break;
	  break;
	}
    case nilEvent:
      /* Collect the data and display it */
      UIBlink ( 140 , 150 );
      if ( Data.CntRSSI == 0 ) 
	FldPrintF ( StatRSSIAvg , "XX"  )
	  else
	    FldPrintF ( StatRSSIAvg , "%d" , -113 + Data.TtlRSSI / Data.CntRSSI )
	    FldPrintF ( StatRSSIMin , "%d" , -113 + Data.MinRSSI )
	    FldPrintF ( StatRSSIMax , "%d" , -113 + Data.MaxRSSI )

	   if ( Data.CntBLER == 0 )
		FldPrintF ( StatBLERAvg , "XX" )
	     else
	        FldPrintF ( StatBLERAvg , "%d" ,  Data.TtlBLER / Data.CntBLER )

      FldPrintF ( StatBLERMin , "%d" ,  Data.MinBLER );
      FldPrintF ( StatBLERMax , "%d" ,  Data.MaxBLER );

      if ( Data.CntrBLER == 0 )
	FldPrintF ( StatrevBLERAvg , "XX" )
      else
	FldPrintF ( StatrevBLERAvg , "%d" ,  Data.TtlrBLER / Data.CntrBLER )

      FldPrintF ( StatrevBLERMin , "%d" ,  Data.MinrBLER );
      FldPrintF ( StatrevBLERMax , "%d" ,  Data.MaxrBLER );

      if ( Data.CntMDBusy == 0 )
	FldPrintF ( StatMDBusyAvg , "XX" )
      else
	   FldPrintF ( StatMDBusyAvg , "%d" ,  Data.TtlMDBusy / Data.CntMDBusy );
      FldPrintF ( StatMDBusyMin , "%d" ,  Data.MinMDBusy );
      FldPrintF ( StatMDBusyMax , "%d" ,  Data.MaxMDBusy );

      FldPrintF ( StatNoSignal , "%d" ,  Data.NoSignl );

      break;
    
    }

  CALL_EPILOGUE;
  return handled;
}

static Boolean ModelRevFormHandleEvent (EventPtr eventP)
{
  Boolean handled;
  FormPtr frmP;

  CALL_PROLOGUE 
    handled = false;

  switch (eventP->eType) 
    {
    case frmOpenEvent:
      frmP = FrmGetActiveForm();
      FrmDrawForm ( frmP);
      Data.ready = -1;
      MinstrelIDRequest  ( &Data );
      handled = true;
      break;
    case ctlSelectEvent:
      frmP = FrmGetActiveForm();
      switch ( eventP->data.ctlSelect.controlID )
	{
	case ChgFormLeft:
	  FrmGotoForm ( MainExpForm );
	  break;
	case ChgFormRight:
	  FrmGotoForm ( StatForm );
	  break;
	case ModelRevReqID:
	  MinstrelIDRequest  ( &Data );
	  break;
	}
    case nilEvent:
      /* Collect the data and display it */
      UIBlink ( 140 , 150 );
      {
	char Tmp[128];
	DumpStrExt ( Tmp , Data.EID , 6 , 0 );
	Say ( ModelRevEID , Tmp );
      }
      
      // Show Model and Revision
      FldPrintF ( ModelRevModel    , "%s" , Data.Model );
      FldPrintF ( ModelRevRevision , "%s" , Data.Revision );
      break;
    
    }

  
  CALL_EPILOGUE;
  return handled;
}


static Boolean MainFormHandleEvent(EventPtr eventP)
{
  Boolean handled;
  FormPtr frmP;


  CALL_PROLOGUE 
    
    handled = false;
  
  switch (eventP->eType) 
    {
      
    case frmOpenEvent:
      frmP = FrmGetActiveForm();
      FrmDrawForm ( frmP);
      Data.ready = -1;
      handled = true;
      break;
      
    case ctlSelectEvent:
      frmP = FrmGetActiveForm();
      if ( ! networkOpen ) PrepareNetwork (); else 
	switch ( eventP->data.ctlSelect.controlID )
	  {
	  case ChgFormRight:
	    if ( Data.ExpForm ) FrmGotoForm ( ModelRevForm );
  	    else FrmGotoForm ( MainExpForm );
   	    break;
	  case ChgFormLeft:
  	    FrmGotoForm ( MainForm );
	  case ctlSelectEvent:
	    frmP = FrmGetActiveForm();
	    break;
	  case MainFlashLED:
	    MinstrelFlashLED ( &Data );
	    break;
	  case MainClearSNDCP:
	    MinstrelClearSNDCP ( &Data );
	    break;
	  case MainReqSNDCP:
	    TestSend (30);
	    break;
	  case MainStatusReq:
	    MinstrelStatusRequest ( &Data );
	    break;
	  case MainRequestID:
	    MinstrelIDRequest ( &Data );
	    break;
	  case MainRequestAuto:
	    MinstrelAutoRequest ( &Data,  1 , 90 );
	    break;
	  default:
	    break;
	  }	
      handled = true;
    case nilEvent:
      /* Collect the data and display it */
      UIBlink ( 140 , 150 );
      if ( Data.ready )
	{
	  char Tmp[128];

          if ( !Data.ExpForm )
	    {  // Shrunk Entries

	      // Show EID
	      DumpStrExt ( Tmp , Data.EID , 6 , 0 );
	      Say ( MainEID , Tmp );
	  
	      // Show Model and Revision
	      FldPrintF ( MainRevModel , "%s(%s)" , Data.Model , Data.Revision );

	      // Show CDPD State
	      FldPrintF ( MainCDPDState, "%s%s%s" ,
			  Data.CDPDStatus & 1 ? "Ch " : " " ,
			  Data.CDPDStatus & 2 ? "Lnk " : " " ,
			  Data.CDPDStatus & 4 ? "Reg " : " " );
	      // Show RRM
	      if ( Data.RRMState < 0 || Data.RRMState > 9 ) 
		Say ( MainRRM , "Bad RRM" );
	      else
		Say ( MainRRM , RRMStateStr[Data.RRMState]);
	      // ServiceID
	      FldPrintF ( MainServID , "%u/%u/%u" , Data.Spi,Data.Wasi,Data.SPNI);
	      FldPrintF ( MainChannel   , "%d" , Data.Channel );
	      FldPrintF ( MainRegErr    , "%d" , Data.RegistrationError );
	      FldPrintF ( MainColor     , "%d" , Data.ColorCode );
	      FldPrintF ( MainPower     , "%d" , Data.PowerLvl );
	      FldPrintF ( MainPowerProd , "%d" , Data.PowerProduct );


	    } 
	  else
	    {  // Expanded entries
	      FldPrintF ( MainSPI    , "%u" , Data.Spi  );
	      FldPrintF ( MainWASI   , "%u" , Data.Wasi );
	      FldPrintF ( MainSPNI   , "%u" , Data.SPNI );
	      FldPrintF ( MainMDBusy , "%d" , Data.MDBSBusy );
	      FldPrintF ( MainUpdNum , "%d" , Data.UpdNum );
	      FldPrintF ( MainRxPack , "%u" , Data.RxPack );
	      FldPrintF ( MainTxPack , "%u" , Data.TxPack );
	      FldPrintF ( MainRxByte , "%lu" , Data.RxBytes );
	      FldPrintF ( MainTxByte , "%lu" , Data.TxBytes );
	      FldPrintF ( MainBatt   , "%d.%dV",Data.VoltageHigh,Data.VoltageLow);
	    }


	  FldPrintF ( MainCellID    , "%u" , Data.CellID );
	  FldPrintF ( MainBLER      , "%d" , Data.BLER );
	  FldPrintF ( MainRevBLER   , "%d" , Data.rBLER );
	  FldPrintF ( MainRSSI      , "%d" , -113 + (int)Data.RSSI );

	  UIBlink ( 40 , 50 );
	}
      break;
    }
  
  CALL_EPILOGUE;
  return handled;
}

static void ReportNetworkError ( int errno )
{
  char Tmp[128];
  StrPrintF ( Tmp , "Network error %d " , 
	      ( ~ netErrorClass ) & errno );
  if ( 47 != (( ~ netErrorClass ) & errno) )	
    Say ( MainStatus , Tmp );
}

		  
void DumpStrExt ( char *Tmp , char * Dta , int n , int spaces )
{
  int i;
  int Step;
  
  Step = spaces ? 3 : 2 ;

  for ( i = 0 ; i < n ; i++ )
    {
      Tmp[i*Step]   = (Dta[i] & 0xf0) / 0x10 + '0';
      Tmp[i*Step+1] = (Dta[i] & 0xf) + '0';
      Tmp[i*Step+2] = ' ';
      if ( Tmp[i*Step+0] > '9' ) Tmp[i*Step+0] += 'A'-'9'-1;
      if ( Tmp[i*Step+1] > '9' ) Tmp[i*Step+1] += 'A'-'9'-1;
    }
  Tmp[n*Step] = 0;
}

void DumpStr ( char * Tmp , char * Dta )
{
  DumpStrExt (  Tmp , Dta , 12 , 1);
}
      
static Boolean ApplicationHandleEvent( EventPtr eventP)
{
  Word formId;
  FormPtr frmP;

  switch (eventP->eType )
    {
    case nilEvent :
      // Process nill event
      if ( ! networkOpen ) { if ( Data.ready ) PrepareNetwork ();}
      else if (MinstrelDataAvailable(&Data))
	{
	  Word len;
	  static int cnt = 0;
	  char Dta[128], Tmp[128];
	  int i;
	    
	  for ( i = 0 ; i < 100 ; i++ ) Dta[i] = 0;
	  len =   MinstrelRecvData ( &Data , Dta , 120 );
	  MinstrelProcessDataPacket ( &Data , Dta );
	    
	  StrPrintF ( Tmp  , "L %d E %d " , len ,errno );
	  Say ( MainStatus , Tmp );
	  DumpStr ( Tmp , Dta    ); Say (MainDump1 , Tmp );
	  DumpStr ( Tmp , Dta+12 ); Say (MainDump2 , Tmp );
	  DumpStr ( Tmp , Dta+12 ); Say (MainDump3 , Tmp );
	}
      break;
    case frmLoadEvent:
      if (eventP->eType == frmLoadEvent)
	{
	  // Load the form resource.
	  formId = eventP->data.frmLoad.formID;
	  frmP = FrmInitForm(formId);
	  FrmSetActiveForm(frmP);
          Data.ExpForm = 0;
	  // Set the event handler for the form.  The handler of the currently
	  // active form is called by FrmHandleEvent each time is receives an
	  // event.
	  switch (formId)
	    {
	    case MainExpForm:
	      Data.ExpForm = 1;
	    case MainForm:
	      FrmSetEventHandler(frmP, MainFormHandleEvent);
	      break;
	    case ModelRevForm:
	      FrmSetEventHandler(frmP, ModelRevFormHandleEvent);
	      break;
	    case StatForm:
	      FrmSetEventHandler(frmP, StatFormHandleEvent);
	      break;
	    case GraphForm:
	      FrmSetEventHandler(frmP, GraphFormHandleEvent);
	      break;
	    default:
	      break;

	    }
	  return true;
	}
    }	
  return false;
}



/***********************************************************************/

static Err StartApplication (void)
{
  Word	 error;			

  Data.ready = 0;
  RestorePreferences ();
  FrmGotoForm ( MainForm );

  return 0;
}





static void StopApplication(void)
{
  SavePreferences ();
  if ( networkOpen ) {
    close (Data.MinSocket); NetLibClose ( AppNetRefnum , -1 ); }
}   



static void EventLoop(void)
{
  EventType event;
  Word      error;

  do
    {
      int timeout = 9;

      EvtGetEvent(&event, timeout);
      if (SysHandleEvent(&event))
	continue;

      if (ApplicationHandleEvent(&event))
	continue;
      FrmDispatchEvent(&event);
    }
  while (event.eType != appStopEvent);
  
}



DWord PilotMain(Word cmd, Ptr cmdBPB, Word launchFlags)
{
  if (cmd == sysAppLaunchCmdNormalLaunch)
    {
      // _gdb_hook ();
      StartApplication();
      EventLoop();
      StopApplication ();
      return(0);
    }
  return(0);
}

