/************************************************************
Module	: Minstrel (Pilot Library)
File	: Minstrel.c
Author	: Vlad Korolev
Contains: Routines for Minstrel Palm  Library
************************************************************/

#include <Pilot.h> 
#include <SysEvtMgr.h> 
#include <NetMgr.h>
#include <Unix/sys_types.h>
#include <Unix/sys_socket.h>
#include <StringMgr.h>
#include "Minstrel.h"

////////////////////////////////////////////
/// String Tables
////////////////////////////////////////////

char  *RRMStateStr [] =
		{
		   "Inactive",
		   "Wide Scan",
		   "Wide Search",
		   "Qual chk",
		   "Chnl Acqd",
		   "Ref Scan",
		   "Chnl Srch",
		   "Dir Hop",
		   "Sleep",
	           "RSSIScan"
		};

char  *MinErrorTbl  [] =
		{
		    "Everything is OK"
		    "No socket available",
		    "Bind errror",
		    "Connect Failed"
		};


int MinstrelInit ( MinstrelData * Dta )
{
  NetIPAddr addr;

  MemSet ( Dta , sizeof (MinstrelData),0);


  Dta->Local.sin_family = AF_INET;
  Dta->Local.sin_port = htons (0);
  Dta->Local.sin_addr.s_addr = 0;

  Dta->Minstrel.sin_family = AF_INET;
  Dta->Minstrel.sin_port = htons (4950);

  addr = NetLibAddrAToIN(AppNetRefnum ,
			  "223.255.255.255" );
  MemMove ( (char *) &Dta->Minstrel.sin_addr.s_addr, &addr, 4);
		    
  if((Dta->MinSocket = 
	socket (AF_INET, SOCK_DGRAM, 17)) < 0) return MinErrorNoSocket;


  if(  bind ( Dta->MinSocket , &Dta->Local , sizeof (Dta->Minstrel)) < 0 )
    {
      close (Dta->MinSocket);
      return MinErrorBindFailed;
    }

  if ( -1 == 
       connect ( Dta->MinSocket , &Dta->Minstrel , sizeof (Dta->Minstrel)))
    {
      close (Dta->MinSocket );
      return MinErrorConnectFailed;
    }

  Dta->ready = 1;

  return 0;
}

int MinstrelProcessDataPacket (MinstrelData * Dta , char * Buf )
{
	    switch ( Buf[0])
	      {
	      case 1: /* Got ID Block */
		MemMove  ( Dta->EID , Buf+1 , 6 );
		{
		  int i,l;
		  for ( i = 7 ; i < 47 && Buf[i] != 0 ; i++ )
		      Dta->Model[i-7] = Buf[i];
		  l = i+1;
		  for ( i = 0 ; 
			i   < 40  && 
			  i+l < 100 &&
			  Buf[i+l] != 0; i ++ )
		  Dta->Revision[i] = Buf[i+l];
		  Dta->Revision[i] = 0;
		  break;
		case 0x23:
		  Dta->CDPDStatus = Buf[1];
		  Dta->RegistrationError = Buf[2];
		  Dta->RRMState = Buf[3];
		  Dta->ColorCode = Buf[4];
		  Dta->RSSI = Buf[7];
		  Dta->BLER = Buf[8];
		  Dta->rBLER = Buf[9];
		  Dta->PowerLvl = Buf[10];
		  Dta->PowerProduct = Buf[19];
		  
		  Dta->MDBSBusy    = Buf[33];

		  // Double Bytes
		  // DragonBall does not support word reads
		  // that are not aligned properly, therefore here goes another
		  // Ugly Hack(tm)
		
		  Dta->Channel = ReadWord(5);
		  Dta->Spi =     ReadWord(11);
		  Dta->Wasi =    ReadWord(13);
		  Dta->SPNI =    ReadWord(15);
		  Dta->CellID =  ReadWord(17);
		  Dta->VoltageHigh = Buf[38];
		  Dta->VoltageLow  = Buf[39];
		  
		  MemMove (&Dta->TEI , (Buf+34),4);

		  Dta->RxPack = ReadWord(21);
 		  Dta->TxPack = ReadWord(23);
		
		  MemMove (&Dta->RxBytes , (Buf+25),4);
		  MemMove (&Dta->TxBytes , (Buf+29),4);
		 
		  ///  Now compute statistics
#ifdef WITH_STATS
		  MinstrelUpdateStats 	( Dta );
#endif

		  break;
		case 3:
		  // Single Byte values
		  Dta->CDPDStatus = Buf[1];
		  Dta->RegistrationError = Buf[3];
		  Dta->RRMState = Buf[4];
		  Dta->ColorCode = Buf[5];     
		  Dta->RSSI = Buf[8];
		  Dta->BLER = Buf[9];
		  Dta->PowerLvl = Buf[11];
		  Dta->PowerProduct = Buf[20];
		  
		  // Double Byte Values
		  Dta->Channel = *(Word*)(Buf+6);
		  Dta->Spi =  *(Word*)(Buf+12);
		  Dta->Wasi = *(Word*)(Buf+14);
		  Dta->SPNI = *(Word*)(Buf+16);
		  Dta->CellID = *(Word*)(Buf+18);
		}
	      }
}

int  MinstrelRecvData ( MinstrelData * Dta , char * Buf , int len )
{
    struct sockaddr_in from;
    Word rSize = sizeof(from);

    return recvfrom ( Dta->MinSocket , Buf , len, 0 , &from , &rSize);
}

int  MinstrelDataAvailable ( MinstrelData * Dta  )
{
	NetFDSetType readFDs,writeFDs,exceptFDs;
	Word numFDs;
	Word width;

	netFDZero(&readFDs);
	netFDZero(&writeFDs);
	netFDZero(&exceptFDs);
	netFDSet(sysFileDescStdIn, &readFDs);
	netFDSet(Dta->MinSocket, &readFDs);
	width = sysFileDescStdIn;
	if (Dta->MinSocket > width) width = Dta->MinSocket;

	numFDs = NetLibSelect(AppNetRefnum,
			      width+1,
			      &readFDs, &writeFDs, &exceptFDs,
			      0, &errno);

	return netFDIsSet ( Dta->MinSocket , &readFDs );
}

void MinstrelSendCommand  ( MinstrelData * Dta , char * Cmd , int len )
{
  send ( Dta->MinSocket , Cmd , len , 0 );
}

void MinstrelIDRequest      ( MinstrelData * Dta )
{
  static char Data[1] = {0};
  MinstrelSendCommand ( Dta , Data , sizeof(Data));
}

void MinstrelStatusRequest  ( MinstrelData * Dta )
{
  static char Data[1] = {2};
  MinstrelSendCommand ( Dta , Data , sizeof(Data));
}

void MinstrelClearSNDCP     ( MinstrelData * Dta )
{
  static char Data[1] = {0x21};
  MinstrelSendCommand ( Dta , Data , sizeof(Data));
}

void MinstrelAutoRequest    ( MinstrelData * Dta , char Repeat , char Count )
{
  static char Data[3] = {0x22, 0 , 0};
  Data[1] = Repeat; Data[2] = Count;
  MinstrelSendCommand ( Dta , Data , sizeof(Data));
}

void MinstrelFlashLED ( MinstrelData * Dta )
{
  static char LED, Data[2]     = { 0xb, 0 };
  Data[1] = (LED = (LED ? 0: 1));
  MinstrelSendCommand ( Dta , Data , sizeof(Data));
}

void MinstrelPowerDown ( MinstrelData * Dta )
{
  static char Data[1]     = { 0xc };
  MinstrelSendCommand ( Dta , Data , sizeof(Data));
}

#ifdef WITH_STATS
void MinstrelUpdateStats ( MinstrelData * Dta )
{
   static ULong svdTime;
   ULong  Time;
   Word intVl;
   Dta->UpdNum ++; 

   if ( Dta->RSSI == 0 || Dta->RRMState != 4 ) { Dta->NoSignl++; } else
   StatUpd(RSSI,TtlRSSI,MaxRSSI,MinRSSI,CntRSSI)
   StatUpd(BLER,TtlBLER,MaxBLER,MinBLER,CntBLER)
   StatUpd(MDBSBusy,TtlMDBusy,MaxMDBusy,MinMDBusy,CntMDBusy)
   if ( Dta->rBLER != -1 || Dta->RRMState != 4)
   StatUpd(rBLER,TtlrBLER,MaxrBLER,MinrBLER,CntrBLER)

   Time = TimGetTicks ();
   if ( Dta->MaxIntvl == 0xFF ) { svdTime = Time; }
   else
   {
       intVl = Time - svdTime;   
       Dta->TtlIntvl += intVl;
       Dta->MaxIntvl = Dta->MaxIntvl > intVl ? Dta->MaxIntvl : intVl;
       Dta->MinIntvl = Dta->MinIntvl < intVl ? Dta->MinIntvl : intVl;
   }
}

void MinstrelResetStats ( MinstrelData * Dta )
{
  Dta->CntMDBusy = Dta->CntRSSI = Dta->CntBLER = Dta->CntrBLER = 0;
  Dta->TtlMDBusy = Dta->TtlRSSI = Dta->TtlBLER = Dta->TtlrBLER = Dta->TtlIntvl = 0;
  Dta->MaxMDBusy = Dta->MaxRSSI = Dta->MaxBLER = Dta->MaxrBLER = Dta->MaxIntvl = 0;
  Dta->MinMDBusy = Dta->MinRSSI = Dta->MinBLER = Dta->MinrBLER = Dta->MinIntvl = 0x7F;
  Dta->NoSignl = Dta->UpdNum = 0;
}
#endif

