/**************************************************
Module      : Minstrel Communicatio Library
File        : Minstrel.h
Purpose     : Function and data type declarations
**************************************************/


/* Read word from the specified location,
   this is needed because DragonBall does not allow
   unaligned long reads */
#define ReadWord(c) (Word)(unsigned char)Buf[c+1]|     \
                    ((Word)(unsigned char)Buf[c]<<8) 

/* Update specfied status variable usig min Max and Total */
#define StatUpd(Var,Ttl,Max,Min,Cntr) Dta->Ttl += Dta->Var; \
   Dta->Max =  Dta->Max < Dta->Var ? Dta->Var : Dta->Max; \
   Dta->Min =  Dta->Min > Dta->Var ? Dta->Var : Dta->Min; \
   if ( Dta->Ttl > 30000 ) { Dta->Ttl /= Dta->Cntr; Dta->Cntr=1;} \
   if ( Dta->Min == 0x7F && Dta->Var != Dta->Min ) Dta->Min = Dta->Var; \
   Dta->Cntr++;

typedef struct
{
  char ready;                            /* 1 - when this structure is ready */
  struct sockaddr_in  Minstrel;          /* Socket addresses for Minstrel end */
  struct sockaddr_in  Local;             /*   and local end.                  */
  int MinSocket;

  /* Identification Data */
  char  EID[6];                          /* Modem Electric ID       */
  char  Revision[40];                    /* Modem Revision Number   */
  char  Model   [40];                    /* Modem Model Number      */

  /*  CDPD Data */
  char  CDPDStatus;                      /* SDPD Status field       */

  char  RegistrationError;               /* Last registration error */
  char  RRMState;                        /* State of RRM            */

  char  ColorCode;                       /* Current Area Color Code */
  Word  Channel;                         /* Current Channel Number  */
  char  RSSI;                            /* Current SIGNAL Strength */
  char  BLER;                            /* Block Error Rate        */
  char  rBLER;                           /* Reverse Block Error Rate*/
  char  PowerLvl;                        /* Transmission Power Level*/
  char  PowerProduct;                    /* Power Product           */

  Word  Spi;                             /* Service Provider ID     */
  Word  Wasi;                            /* Wide Area Service ID    */
  Word  SPNI;                            /* Service Provider NetID  */
  Word  CellID;                          /* Current CELL Tower      */

  char  VoltageLow;                      /* Battery Voltage LowByte */
  char  VoltageHigh;                     /* Battery Voltage HighByte*/

  /* SNDCP Data */
  Word  RxPack;                          /* SNDCP Packets recieved   */
  Word  TxPack;                          /* SNDCP Packet  transmitted*/
  DWord RxBytes;                         /* Bytes Recieved           */
  DWord TxBytes;                         /* Bytes Transmitted        */
  
  char  MDBSBusy;                        /* Percentage MDBS Busy     */
  DWord TEI;


  /* Now goes statistical data */
#ifdef WITH_STATS
  int   UpdNum;                           /* Number of points accumulated */        
 
  Word   NoSignl;

  Word   CntMDBusy  ;                       /* Number of valid MDBusy measurements */
  Word   CntRSSI  ;                       /* Number of valid RSSI measurements */
  Word   CntBLER  ;                       /* Number of valid rBLER measurements */
  Word   CntrBLER ;                       /* Number of valid BLER measurements */

  Word   TtlMDBusy;                       /* Sum of MDBusy */
  Word   TtlRSSI;                         /* Sum of RSSI */
  Word   TtlBLER;                         /* Sum of BLER */
  Word   TtlrBLER;                        /* Sum of rBLER */
  Word   TtlIntvl;                        /* Total interval (not valid)*/

  char   MinMDBusy;                         /* Minimum MDBusy */
  char   MinRSSI;                         /* Minimum RSSI */
  char   MinBLER;                         /* Minimum BLER */
  char   MinrBLER;                        /* Minimum rBLER */
  char   MinIntvl;                        /* Min Interval (sucks) */
  
  char   MaxMDBusy;                         /* Minimum MDBusy */
  char   MaxRSSI;                         /* Minimum RSSI */
  char   MaxBLER;                         /* Minimum BLER */
  char   MaxrBLER;                        /* Minimum rBLER */
  unsigned char   MaxIntvl;

#endif
 
  int    ExpForm;                         /* This should not be here, but
					     I will leave it for user's data */

} MinstrelData;


extern char  *RRMStateStr [] ;
extern char  *MinErrorTbl  [] ;


typedef enum { MinErrorNoSocket = 1,
	       MinErrorBindFailed,
	       MinErrorConnectFailed  }  MinstrelError ;


int  MinstrelRecvData         ( MinstrelData * Dta , char * Buf , int len ); 
int  MinstrelProcessDataPacket( MinstrelData * Dta , char * Buf );
int  MinstrelInit             ( MinstrelData * Dta );
void MinstrelPowerDown        ( MinstrelData * Dta );
void MinstrelFlashLED         ( MinstrelData * Dta );
void MinstrelAutoRequest      ( MinstrelData * Dta , char Repeat , char Count );
void MinstrelClearSNDCP       ( MinstrelData * Dta );
void MinstrelStatusRequest    ( MinstrelData * Dta );
void MinstrelIDRequest        ( MinstrelData * Dta );
void MinstrelSendCommand      ( MinstrelData * Dta , char * Cmd , int len );
int  MinstrelDataAvailable    ( MinstrelData * Dta );

#ifdef WITH_STATS
void MinstrelResetStats       ( MinstrelData * Dta );
void MinstrelUpdateStats      ( MinstrelData * Dta );
#endif
